#include <QApplication>
#include <QFontMetrics>
#define private protected
#include <QLineEdit>
#undef private
#include <QPainter>
#include <QPixmap>
#include <QStyleOptionFrame>
#include <QTextLine>
#include <QDebug>

#include "qlineedit_p.h"

#define innerMargin 1

class KSearchLineEdit : public QLineEdit
{
    Q_OBJECT

    public:
        KSearchLineEdit(QWidget* parent = 0)
            : QLineEdit(parent)
        {
            init();
        }

        KSearchLineEdit(const QString& contents, QWidget* parent)
            :QLineEdit(contents, parent)
        {
            init();
        }

    protected:
        QPixmap m_pm;
        void init()
        {
            // FIXME: HACK! we'd actually use kiconloader here and aim for a size
            // that matches our size... e.g. reimp resizeEvent and load if necessary there
            m_pm.load("clear_left.png", "png");
        }

        QStyleOptionFrame getStyleOption() const
        {
            QStyleOptionFrame opt;
            opt.init(this);
            opt.lineWidth = hasFrame() ? style()->pixelMetric(QStyle::PM_DefaultFrameWidth) : 0;
            opt.midLineWidth = 0;
            opt.state |= QStyle::State_Sunken;
            if (isReadOnly())
                opt.state |= QStyle::State_ReadOnly;
#ifdef QT_KEYPAD_NAVIGATION
            if (q->hasEditFocus())
                opt.state |= QStyle::State_HasEditFocus;
#endif
            return opt;
        }

        void paintEvent(QPaintEvent *)
        {
            Q_D(QLineEdit);
            QPainter p(this);

            QRect r = rect();
            const QPalette &pal = palette();

            QStyleOptionFrame panel = getStyleOption();
            style()->drawPrimitive(QStyle::PE_PanelLineEdit, &panel, &p, this);

            if (hasFrame()) {
                int frameWidth = panel.lineWidth;
                r.adjust(frameWidth, frameWidth, -frameWidth, -frameWidth);
                p.setClipRect(r);
            }

            QFontMetrics fm = fontMetrics();
            QRect lineRect(r.x() + innerMargin, r.y() + (r.height() - fm.height() + 1) / 2,
                           r.width() - 3*innerMargin - m_pm.width(), fm.height());
            QTextLine line = d->textLayout.lineAt(0);

            // locate cursor position
            int cix = qRound(line.cursorToX(d->cursor));

            // horizontal scrolling
            int minLB = qMax(0, -fm.minLeftBearing());
            int minRB = qMax(0, -fm.minRightBearing());


            int widthUsed = qRound(line.naturalTextWidth()) + 1 + minRB;
            if ((minLB + widthUsed) <=  lineRect.width()) {
                Qt::Alignment va = QStyle::visualAlignment(layoutDirection(), QFlag(d->alignment));
                va &= ~(Qt::AlignAbsolute|Qt::AlignVertical_Mask);
                switch (va) {
                    case Qt::AlignRight:
                        d->hscroll = widthUsed - lineRect.width() + 1;
                        break;
                    case Qt::AlignHCenter:
                        d->hscroll = (widthUsed - lineRect.width()) / 2;
                        break;
                    default:
                        // Left
                        d->hscroll = 0;
                        break;
                }
                d->hscroll -= minLB;
            } else if (cix - d->hscroll >= lineRect.width()) {
                d->hscroll = cix - lineRect.width() + 1;
            } else if (cix - d->hscroll < 0) {
                d->hscroll = cix;
            } else if (widthUsed - d->hscroll < lineRect.width()) {
                d->hscroll = widthUsed - lineRect.width() + 1;
            }
            // the y offset is there to keep the baseline constant in case we have script changes in the text.
            QPoint topLeft = lineRect.topLeft() - QPoint(d->hscroll, d->ascent-fm.ascent());

            // draw text, selections and cursors
            p.setPen(pal.text().color());
            bool supressCursor = isReadOnly();

            QVector<QTextLayout::FormatRange> selections;
            if (d->selstart < d->selend || (d->cursorVisible && d->maskData)) {
                QTextLayout::FormatRange o;
                const QPalette &pal = palette();
                if (d->selstart < d->selend) {
                    o.start = d->selstart;
                    o.length = d->selend - d->selstart;
                    o.format.setBackground(pal.brush(QPalette::Highlight));
                    o.format.setForeground(pal.brush(QPalette::HighlightedText));
                } else {
                    // mask selection
                    o.start = d->cursor;
                    o.length = 1;
                    o.format.setBackground(pal.brush(QPalette::Text));
                    o.format.setForeground(pal.brush(QPalette::Background));
                }
                selections.append(o);
            }

            // Asian users see an IM selection text as cursor on candidate
            // selection phase of input method, so the ordinary cursor should be
            // invisible if we have a preedit string.
            bool showCursor = (d->cursorVisible && !supressCursor && !d->textLayout.preeditAreaText().length());
            d->textLayout.draw(&p, topLeft, selections, r);
            if (showCursor && d->textLayout.preeditAreaText().isEmpty())
                d->textLayout.drawCursor(&p, topLeft, d->cursor);

            p.drawPixmap(lineRect.width() + innerMargin, (height() - m_pm.height()) / 2, m_pm);
        }
};

int main(int argc, char* argv[])
{
    QApplication app(argc, argv);
    KSearchLineEdit* line = new KSearchLineEdit(0);
    line->show();
    return app.exec();
}

#include "searchline.moc"
